#include "include/ttail.h"

ttail_t *ttail_init(int argc, char **argv)
{
	ttail_t *res;
	int c, opt_i, ret;

	res = malloc(sizeof(ttail_t));
	if(!res)
	{
		perror("Unable to allocate memory");
		goto ttail_init_alloc_err;
	}

	opterr = 0;
	res->verbose = 0;
	res->fmt = NULL;
	regfree(&(res->date_prefix));
	res->date_prefix_flag = 0;
	res->logfile = NULL;
	res->logfile_name = NULL;
	res->logfile_sz = 0;

	while(1)
	{
		static struct option long_options[] =
		{
			{"verbose", no_argument, 0, 'v'},
			{"re-prefix", required_argument, 0, 'p'},
			{"date-format", required_argument, 0, 'd'},
			{"file", required_argument, 0, 'f'},
		};
		opt_i = 0;
		c = getopt_long(argc, argv, "p:f:d:", long_options, &opt_i);
		if(c == -1)
			break;
		
		switch(c)
		{
			case 'v':
				res->verbose++;
				break;
			case 'p':
				if(ttail_set_prefix(res, optarg) < 0)
				{
					goto ttail_init_err;
				}
				break;
			case 'd':
				if(res->fmt)
				{
					fprintf(stderr,"Warning : multiple\
date format given\n");
				}
				res->fmt = malloc(
					sizeof(char)*(strlen(optarg)+1));
				if(!res->fmt)
				{
					perror("Unable to allocate memory\
for date format");
					goto ttail_init_err;
				}
				strcpy(res->fmt, optarg);

				break;
			case 'f':
				ret = ttail_add_logfile(res, optarg);
				if(ret < 0)
				{
					goto ttail_init_err;
				}
				break;
		}
	}

	return res;

	ttail_init_err:
	ttail_free(res);
	ttail_init_alloc_err:
	return NULL;
}

int ttail_add_logfile(ttail_t* res, const char* filename)
{
	void *tmp;
	FILE *fp;
	char *fname;
	res->logfile_sz++;
	tmp = res->logfile;
	res->logfile = realloc(res->logfile,
		sizeof(FILE*)*res->logfile_sz);
	if(!res->logfile)
	{
		perror("Unable to allocate memory for logfiles");
		res->logfile = tmp;
		goto ttail_add_logfile_fpalloc_err;
	}
	fp = fopen(optarg, "r");
	if(!fp)
	{
		fprintf(stderr, "Unable to open file : %s", filename);
	}
	res->logfile[res->logfile_sz-1] = fp;

	tmp = res->logfile_name;
	res->logfile_name = realloc(res->logfile_name,
		sizeof(char*)*res->logfile_sz);
	if(!res->logfile_name)
	{
		perror("Unable to allocate memory for logfiles");
		res->logfile_name = tmp;
		goto ttail_add_logfile_fnalloc_err;

	}
	fname = malloc(sizeof(char)*(strlen(filename)+1));
	if(!fname)
	{
		perror("Unable to allocate memory for logfiles");
		goto ttail_add_logfile_fnalloc_err;
	}
	strcpy(fname, filename);
	res->logfile_name[res->logfile_sz-1] = fname;
	
	return 0;

	ttail_add_logfile_fnalloc_err:
	fclose(res->logfile[res->logfile_sz-2]);
	ttail_add_logfile_fpalloc_err:
	res->logfile_sz--;
	return -1;
}

int ttail_set_prefix(ttail_t* res, const char* regex)
{
	int ret;
	char *re_errbuff;
	size_t re_errbuff_sz;

	if(res->date_prefix_flag)
	{
		fprintf(stderr, "Warning : Regex prefix allready set");
		return 0;
	}
	res->date_prefix_flag = 1;
	ret = regcomp(&res->date_prefix, regex, 0);
	if(!ret)
	{
		return 0;
	}
	/*compilation error */
	re_errbuff_sz = regerror(ret,
		&res->date_prefix, NULL, 0);
	re_errbuff = malloc(
		sizeof(char)*re_errbuff_sz);
	if(!re_errbuff)
	{
		perror("Failed to allocate memory for regex compilation \
error message");
		goto ttail_set_prefix_err;
	}
	regerror(ret, &res->date_prefix,
		re_errbuff,
		sizeof(char)*re_errbuff_sz);
	fprintf(stderr, "Regex compilation fails : %s", re_errbuff);

	free(re_errbuff);
	ttail_set_prefix_err:
	return -1;
}

void ttail_free(ttail_t* t)
{
	size_t i;
	for(i=0; i<t->logfile_sz; i++)
	{
		if(t->logfile[i])
		{
			fclose(t->logfile[i]);
		}
		if(t->logfile_name[i])
		{
			free(t->logfile_name[i]);
		}
	}
	t->logfile_sz = 0;
	if(t->fmt != NULL)
	{
		free(t->fmt);
	}
	free(t);
}

